require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xvhd',
  dscmeta_resource_friendly_name: 'xVHD',
  dscmeta_resource_name: 'MSFT_xVHD',
  dscmeta_module_name: 'xHyper-V',
  dscmeta_module_version: '3.4.0.0',
  docs: 'The DSC xVHD resource type.
         Automatically generated from version 3.4.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_filesizebytes: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'Current size of the VHD',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_generation: {
      type: "Optional[Enum['Vhd', 'Vhdx']]",
      desc: 'Virtual disk format - Vhd or Vhdx',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_type: {
      type: 'Optional[String]',
      desc: 'Type of Vhd - Dynamic, Fixed, Differencing',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_id: {
      type: 'Optional[String]',
      desc: 'Virtual Disk Identifier',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Should the VHD be created or deleted',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_maximumsizebytes: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'Maximum size of Vhd to be created',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Name of the VHD File',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_isattached: {
      type: 'Optional[Boolean]',
      desc: 'Is the VHD attached to a VM or not',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_path: {
      type: 'String',
      desc: 'Folder where the VHD will be created',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_parentpath: {
      type: 'Optional[String]',
      desc: 'Parent VHD file path, for differencing disk',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
