require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xvhdfile',
  dscmeta_resource_friendly_name: 'xVhdFile',
  dscmeta_resource_name: 'MSFT_xVhdFileDirectory',
  dscmeta_module_name: 'xHyper-V',
  dscmeta_module_version: '3.4.0.0',
  docs: 'The DSC xVhdFile resource type.
         Automatically generated from version 3.4.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_filedirectory: {
      type: "Array[Struct[{
              type => Optional[Enum['File', 'Directory']],
              ensure => Optional[Enum['Present', 'Absent']],
              content => Optional[String],
              destinationpath => String,
              force => Optional[Boolean],
              sourcepath => Optional[String],
              attributes => Optional[Array[Enum['ReadOnly', 'Hidden', 'System', 'Archive']]],
              recurse => Optional[Boolean]
            }]]",
      desc: 'The FileDirectory objects to copy to the VHD',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'MSFT_xFileDirectory[]',
      mof_is_embedded: true,
    },
    dsc_checksum: {
      type: "Optional[Enum['ModifiedDate', 'SHA-1', 'SHA-256', 'SHA-512']]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_vhdpath: {
      type: 'String',
      desc: 'Path to the VHD',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
