require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xvmhyperv',
  dscmeta_resource_friendly_name: 'xVMHyperV',
  dscmeta_resource_name: 'MSFT_xVMHyperV',
  dscmeta_module_name: 'xHyper-V',
  dscmeta_module_version: '3.4.0.0',
  docs: 'The DSC xVMHyperV resource type.
         Automatically generated from version 3.4.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_creationtime: {
      type: 'Optional[Timestamp]',
      desc: 'Creation time of the VM',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'DateTime',
      mof_is_embedded: false,
    },
    dsc_vhdpath: {
      type: 'String',
      desc: 'VHD associated with the VM',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_id: {
      type: 'Optional[String]',
      desc: 'VM unique ID',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_startupmemory: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'Startup RAM for the VM.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Name of the VM',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_minimummemory: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'Minimum RAM for the VM. This enables dynamic memory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_waitforip: {
      type: 'Optional[Boolean]',
      desc: 'Waits for VM to get valid IP address.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_memoryassigned: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'Memory assigned to the VM',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_macaddress: {
      type: 'Optional[Array[String]]',
      desc: 'MAC address(es) of the VM NICs.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_maximummemory: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'Maximum RAM for the VM. This enable dynamic memory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_notes: {
      type: 'Optional[String]',
      desc: 'Notes about the VM.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_path: {
      type: 'Optional[String]',
      desc: 'Folder where the VM data will be stored',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Should the VM be created or deleted',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_restartifneeded: {
      type: 'Optional[Boolean]',
      desc: 'If specified, shutdowns and restarts the VM as needed for property changes',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_hasdynamicmemory: {
      type: 'Optional[Boolean]',
      desc: 'Does VM has dynamic memory enabled',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_processorcount: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Processor count for the VM',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_networkadapters: {
      type: 'Optional[Array[String]]',
      desc: "Network adapters' IP addresses of the VM",
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_secureboot: {
      type: 'Optional[Boolean]',
      desc: 'Enable secure boot for Generation 2 VMs.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_state: {
      type: "Optional[Enum['Running', 'Paused', 'Off']]",
      desc: 'State of the VM.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_switchname: {
      type: 'Optional[Array[String]]',
      desc: 'Virtual switch(es) associated with the VM',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_generation: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Virtual machine generation',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_uptime: {
      type: 'Optional[String]',
      desc: 'Uptime of the VM',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_enableguestservice: {
      type: 'Optional[Boolean]',
      desc: 'Enable Guest Service Interface for the VM.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_status: {
      type: 'Optional[String]',
      desc: 'Status of the VM',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_cpuusage: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'CPU Usage of the VM',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
  },
)
