require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xvmswitch',
  dscmeta_resource_friendly_name: 'xVMSwitch',
  dscmeta_resource_name: 'MSFT_xVMSwitch',
  dscmeta_module_name: 'xHyper-V',
  dscmeta_module_version: '3.4.0.0',
  docs: 'The DSC xVMSwitch resource type.
         Automatically generated from version 3.4.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_type: {
      type: "Enum['External', 'Internal', 'Private']",
      desc: 'Type of switch',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_id: {
      type: 'Optional[String]',
      desc: 'Unique ID for the switch',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Whether switch should be present or absent',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allowmanagementos: {
      type: 'Optional[Boolean]',
      desc: 'Specify if the VM host has access to the physical NIC',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Name of the VM Switch',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_netadaptername: {
      type: 'Optional[String]',
      desc: 'Network adapter name for external switch type',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_bandwidthreservationmode: {
      type: "Optional[Enum['Default', 'Weight', 'Absolute', 'None', 'NA']]",
      desc: 'Type of Bandwidth Reservation Mode to use for the switch',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_netadapterinterfacedescription: {
      type: 'Optional[String]',
      desc: 'Description of the network interface',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
